# -*- coding: utf-8 -*-

translations = {
    # Days
    'days': {
        0: 'Bazar',
        1: 'Bazar ertəsi',
        2: 'Çərşənbə axşamı',
        3: 'Çərşənbə',
        4: 'Cümə axşamı',
        5: 'Cümə',
        6: 'Şənbə'
    },
    'days_abbrev': {
        0: 'Baz',
        1: 'BzE',
        2: 'ÇAx',
        3: 'Çər',
        4: 'CAx',
        5: 'Cüm',
        6: 'Şən'
    },

    # Months
    'months': {
        1: 'yanvar',
        2: 'fevral',
        3: 'mart',
        4: 'aprel',
        5: 'may',
        6: 'iyun',
        7: 'iyul',
        8: 'avqust',
        9: 'sentyabr',
        10: 'oktyabr',
        11: 'noyabr',
        12: 'dekabr',
    },
    'months_abbrev': {
        1: 'yan',
        2: 'fev',
        3: 'mar',
        4: 'apr',
        5: 'may',
        6: 'iyn',
        7: 'iyl',
        8: 'avq',
        9: 'sen',
        10: 'okt',
        11: 'noy',
        12: 'dek',
    },

    # Units of time
    'year': '{count} il',
    'month': '{count} ay',
    'week': '{count} həftə',
    'day': '{count} gün',
    'hour': '{count} saat',
    'minute': '{count} dəqiqə',
    'second': '{count} saniyə',

    # Relative time
    'ago': '{time} öncə',
    'from_now': '{time} sonra',
    'after': '{time} sonra',
    'before': '{time} öncə',

    # Date formats
    'date_formats': {
        'LTS': 'HH:mm:ss',
        'LT': 'HH:mm',
        'LLLL': 'dddd, D MMMM YYYY HH:mm',
        'LLL': 'D MMMM YYYY HH:mm',
        'LL': 'D MMMM YYYY',
        'L': 'DD.MM.YYYY',
    },
}



# -*- coding: utf-8 -*-


class TransitionType(object):

    def __init__(self, utc_offset, is_dst, abbrev):
        self.utc_offset = utc_offset
        self.is_dst = is_dst
        self.abbrev = abbrev

    def __repr__(self):
        return '<TransitionType [{}, {}, {}]>'.format(
            self.utc_offset,
            self.is_dst,
            self.abbrev
        )



# -*- coding: utf-8 -*-

from datetime import tzinfo, timedelta


class TimezoneInfo(tzinfo):

    def __init__(self, tz, utc_offset, is_dst, dst, abbrev):
        """
        :param tz: The parent timezone.
        :type tz: Timezone

        :param utc_offset: Offset from UTC (in seconds).
        :type utc_offset: int

        :param is_dst: Whether DST is in effect or not.
        :type is_dst: bool

        :param dst: Transition time value (if any).
        :type dst: timedelta or None

        :param abbrev: Timezone name abbreviation (CET, CEST, ...)
        :type abbrev: str
        """
        self._tz = tz
        self._utc_offset = utc_offset
        self._adjusted_offset = timedelta(seconds=round(utc_offset / 60) * 60)
        self._is_dst = is_dst
        self._dst = dst
        self._abbrev = abbrev

    @property
    def tz(self):
        return self._tz

    @property
    def name(self):
        return self._tz._name

    @property
    def offset(self):
        return self._utc_offset

    @property
    def is_dst(self):
        return self._is_dst

    @property
    def abbrev(self):
        return self._abbrev

    @property
    def adjusted_offset(self):
        return self._adjusted_offset

    @property
    def dst_(self):
        return self._dst

    def tzname(self, dt):
        if dt is None:
            return self._abbrev
        elif dt.tzinfo is not self:
            dt = self.tz.convert(dt)

            return dt.tzinfo.abbrev
        else:
            return self._abbrev

    def utcoffset(self, dt):
        if dt is None:
            return None
        elif dt.tzinfo is not self:
            dt = self.tz.convert(dt)

            return dt.tzinfo.adjusted_offset
        else:
            return self._adjusted_offset

    def dst(self, dt):
        if not self.is_dst:
            return None

        if dt is None:
            return None
        elif dt.tzinfo is not self:
            dt = self.tz.convert(dt)

            offset = dt.tzinfo._dst
        else:
            offset = self._dst

        return offset

    def fromutc(self, dt):
        dt = dt.replace(tzinfo=None)

        idx = max(0, self._tz._find_transition_index(dt, '_utc_time') - 1)
        tzinfo = self._tz._tzinfos[self._tz._transitions[idx]._tzinfo_index]

        return (dt + tzinfo.adjusted_offset).replace(tzinfo=tzinfo)

    def __repr__(self):
        return '<TimezoneInfo [{}, {}, {}{}, {}]>'.format(
            self.name,
            self.abbrev,
            '+' if self.offset >= 0 else '',
            '{:02d}:{:02d}:{:02d}'.format(
                self.offset // 3600,
                self.offset % 3600 // 60,
                self.offset % 60
            ),
            'DST' if self.is_dst else 'STD',
        )

    def __getinitargs__(self):
        return self._tz, self._utc_offset, self._is_dst, self._dst, self._abbrev

class _UTC(TimezoneInfo):

    def __init__(self):
        super(_UTC, self).__init__(None, 0, False, None, 'GMT')

    @property
    def name(self):
        return 'UTC'

    def utcoffset(self, dt):
        return self._adjusted_offset

    def dst(self, dt):
        return None

    def fromutc(self, dt):
        return dt.replace(tzinfo=self)

    def __getinitargs__(self):
        return ()


UTC = _UTC()



# -*- coding: utf-8 -*-

translations = {
    # Days
    'days': {
        0: 'sunnudagur',
        1: 'mánadagur',
        2: 'týsdagur',
        3: 'mikudagur',
        4: 'hósdagur',
        5: 'fríggjadagur',
        6: 'leygardagur'
    },
    'days_abbrev': {
        0: 'sun',
        1: 'mán',
        2: 'týs',
        3: 'mik',
        4: 'hós',
        5: 'frí',
        6: 'ley'
    },

    # Months
    'months': {
        1: 'januar',
        2: 'februar',
        3: 'mars',
        4: 'apríl',
        5: 'mai',
        6: 'juni',
        7: 'juli',
        8: 'august',
        9: 'september',
        10: 'oktober',
        11: 'november',
        12: 'desember',
    },
    'months_abbrev': {
        1: 'jan',
        2: 'feb',
        3: 'mar',
        4: 'apr',
        5: 'mai',
        6: 'jun',
        7: 'jul',
        8: 'aug',
        9: 'sep',
        10: 'okt',
        11: 'nov',
        12: 'des',
    },

    # Units of time
    'year': ['{count} ár', '{count} ár'],
    'month': ['{count} mánaður', '{count} mánaðir'],
    'week': ['{count} vika', '{count} vikur'],
    'day': ['{count} dag', '{count} dagar'],
    'hour': ['{count} tími', '{count} tímar'],
    'minute': ['{count} minutt', '{count} minuttir'],
    'second': ['{count} sekund', '{count} sekundir'],

    # Relative time
    'ago': '{time} síðan',
    'from_now': 'um {time}',
    'after': '{time} aftaná',
    'before': '{time} áðrenn',

    # Ordinals
    'ordinal': '.',

    # Date formats
    'date_formats': {
        'LTS': 'HH:mm:ss',
        'LT': 'HH:mm',
        'LLLL': 'dddd D. MMMM, YYYY HH:mm',
        'LLL': 'D MMMM YYYY HH:mm',
        'LL': 'D MMMM YYYY',
        'L': 'DD/MM/YYYY',
    },
}



# -*- coding: utf-8 -*-

from datetime import time, datetime, timedelta

from .interval import Interval, AbsoluteInterval
from .mixins.default import TranslatableMixin, FormattableMixing, TestableMixin
from .constants import (
    USECS_PER_SEC, SECS_PER_HOUR, SECS_PER_MIN
)


class Time(TranslatableMixin, FormattableMixing, TestableMixin, time):
    """
    Represents a time instance as hour, minute, second, microsecond.
    """

    def __init__(self, hour, minute=0, second=0, microsecond=0,
                 tzinfo=None, fold=0):
        """
        Constructor.

        :param hour: The hour.
        :type hour: int

        :param minute: The minute.
        :type minute: int

        :param second: The second
        :type second: int

        :param microsecond: The microsecond
        :type microsecond: int

        :param tzinfo: The timezone info (not used)
        :type tzinfo: tzinfo or None
        """
        self._hour = hour
        self._minute = minute
        self._second = second
        self._microsecond = microsecond
        self._tzinfo = tzinfo
        self._time = time(hour, minute, second, microsecond, tzinfo)
        self._fold = fold

    @classmethod
    def instance(cls, t, copy=True):
        """
        Creates a Time instance from a time object.

        It will raise a TypeError exception if the time
        is timezone aware.

        :param t: The time object
        :type t: time

        :param copy: Whether to return a copy of the instance or not.
        :type copy: bool

        :rtype: Time

        :raises: TypeError
        """
        if isinstance(t, Time) and not copy:
            return t

        return cls(t.hour, t.minute, t.second, t.microsecond, t.tzinfo)

    @classmethod
    def now(cls, with_microseconds=True):
        """
        Return a Time instance corresponding to the current time.

        It will return your local time.

        By default, it will include microseconds.
        Just set ``with_microseconds`` to ``False`` to exclude them.

        :param with_microseconds: Whether to include microseconds or not.
        :type with_microseconds: bool

        :rtype: Time
        """
        if cls.has_test_now():
            if not with_microseconds:
                return cls.get_test_now().replace(microsecond=0)

            return cls.get_test_now()

        now = datetime.now()

        microsecond = now.microsecond if with_microseconds else 0

        return cls(now.hour, now.minute, now.second, microsecond)

    @property
    def hour(self):
        return self._hour

    @property
    def minute(self):
        return self._minute

    @property
    def second(self):
        return self._second

    @property
    def microsecond(self):
        return self._microsecond

    @property
    def tzinfo(self):
        return self._tzinfo

    @property
    def fold(self):
        return self._fold

    # Comparisons

    def between(self, dt1, dt2, equal=True):
        """
        Determines if the instance is between two others.

        :type dt1: Time or time
        :type dt2: Time or time

        :param equal: Indicates if a > and < comparison shoud be used or <= and >=

        :rtype: bool
        """
        if dt1 > dt2:
            dt1, dt2 = dt2, dt1

        if equal:
            return self >= dt1 and self <= dt2

        return self > dt1 and self < dt2

    def closest(self, dt1, dt2):
        """
        Get the closest time from the instance.

        :type dt1: Time or time
        :type dt2: Time or time

        :rtype: Time
        """
        dt1 = self.instance(dt1, False)
        dt2 = self.instance(dt2, False)

        if self.diff(dt1).in_seconds() < self.diff(dt2).in_seconds():
            return dt1

        return dt2

    def farthest(self, dt1, dt2):
        """
        Get the farthest time from the instance.

        :type dt1: Time or time
        :type dt2: Time or time

        :rtype: Time
        """
        dt1 = self.instance(dt1, False)
        dt2 = self.instance(dt2, False)

        if self.diff(dt1).in_seconds() > self.diff(dt2).in_seconds():
            return dt1

        return dt2

    def min_(self, dt=None):
        """
        Get the minimum instance between a given instance (default now)
        and the current instance.

        :type dt: Time or time

        :rtype: Time
        """
        if dt is None:
            dt = Time.now()

        if self < dt:
            return self

        return self.instance(dt, False)

    def minimum(self, dt=None):
        """
        Get the minimum instance between a given instance (default now)
        and the current instance.

        :type dt: Time or time

        :rtype: Time
        """
        return self.min_(dt)

    def max_(self, dt=None):
        """
        Get the maximum instance between a given instance (default now)
        and the current instance.

        :type dt: Time or time

        :rtype: Time
        """
        if dt is None:
            dt = Time.now()

        if self > dt:
            return self

        return self.instance(dt, False)

    def maximum(self, dt=None):
        """
        Get the maximum instance between a given instance (default now)
        and the current instance.

        :type dt: Time or time

        :rtype: Time
        """
        return self.max_(dt)

    def __hash__(self):
        return self._time.__hash__()

    # ADDITIONS AND SUBSTRACTIONS

    def add(self, hours=0, minutes=0, seconds=0, microseconds=0):
        """
        Add duration to the instance.

        :param hours: The number of hours
        :type hours: int

        :param minutes: The number of minutes
        :type minutes: int

        :param seconds: The number of seconds
        :type seconds: int

        :param microseconds: The number of microseconds
        :type microseconds: int

        :rtype: Time
        """
        from .pendulum import Pendulum

        return Pendulum.EPOCH.at(
            self._hour, self._minute, self._second, self._microsecond
        ).add(
            hours=hours,
            minutes=minutes,
            seconds=seconds,
            microseconds=microseconds
        ).time()

    def subtract(self, hours=0, minutes=0, seconds=0, microseconds=0):
        """
        Add duration to the instance.

        :param hours: The number of hours
        :type hours: int

        :param minutes: The number of minutes
        :type minutes: int

        :param seconds: The number of seconds
        :type seconds: int

        :param microseconds: The number of microseconds
        :type microseconds: int

        :rtype: Time
        """
        from .pendulum import Pendulum

        return Pendulum.EPOCH.at(
            self._hour, self._minute, self._second, self._microsecond
        ).subtract(
            hours=hours,
            minutes=minutes,
            seconds=seconds,
            microseconds=microseconds
        ).time()

    def add_timedelta(self, delta):
        """
        Add timedelta duration to the instance.

        :param delta: The timedelta instance
        :type delta: datetime.timedelta

        :rtype: Time
        """
        if delta.days:
            raise TypeError('Cannot timedelta with days to Time.')

        return self.add(
            seconds=delta.seconds,
            microseconds=delta.microseconds
        )

    def subtract_timedelta(self, delta):
        """
        Remove timedelta duration from the instance.

        :param delta: The timedelta instance
        :type delta: datetime.timedelta

        :rtype: Time
        """
        if delta.days:
            raise TypeError('Cannot timedelta with days to Time.')

        return self.subtract(
            seconds=delta.seconds,
            microseconds=delta.microseconds
        )

    def __add__(self, other):
        if not isinstance(other, timedelta):
            return NotImplemented

        return self.add_timedelta(other)

    def __sub__(self, other):
        if not isinstance(other, (Time, time, timedelta)):
            return NotImplemented

        if isinstance(other, timedelta):
            return self.subtract_timedelta(other)

        if isinstance(other, time):
            if other.tzinfo is not None:
                raise TypeError('Cannot subtract aware times to or from Time.')

            other = self.instance(other)

        return other.diff(self, False)

    def __rsub__(self, other):
        if not isinstance(other, (Time, time)):
            return NotImplemented

        if isinstance(other, time):
            if other.tzinfo is not None:
                raise TypeError('Cannot subtract aware times to or from Time.')

            other = self.instance(other)

        return other.__sub__(self)

    # DIFFERENCES

    def diff(self, dt=None, abs=True):
        """
        Returns the difference between two Time objects as an Interval.

        :type dt: Time or None

        :param abs: Whether to return an absolute interval or not
        :type abs: bool

        :rtype: Interval
        """
        if dt is None:
            dt = self.now()
        else:
            dt = self.instance(dt, False)

        us1 = (
            self.hour * SECS_PER_HOUR
            + self.minute * SECS_PER_MIN
            + self.second
        ) * USECS_PER_SEC

        us2 = (
            dt.hour * SECS_PER_HOUR
            + dt.minute * SECS_PER_MIN
            + dt.second
        ) * USECS_PER_SEC

        klass = Interval
        if abs:
            klass = AbsoluteInterval

        return klass(microseconds=us2 - us1)

    def diff_for_humans(self, other=None, absolute=False, locale=None):
        """
        Get the difference in a human readable format in the current locale.

        :type other: Time or time

        :param absolute: removes time difference modifiers ago, after, etc
        :type absolute: bool

        :param locale: The locale to use for localization
        :type locale: str

        :rtype: str
        """
        is_now = other is None

        if is_now:
            other = self.now()

        diff = self.diff(other)

        if diff.hours > 0:
            unit = 'hour'
            count = diff.hours
        elif diff.minutes > 0:
            unit = 'minute'
            count = diff.minutes
        else:
            unit = 'second'
            count = diff.seconds

        if count == 0:
            count = 1

        time = self.translator().transchoice(unit, count, {'count': count}, locale=locale)

        if absolute:
            return time

        is_future = diff.invert

        if is_now:
            trans_id = 'from_now' if is_future else 'ago'
        else:
            trans_id = 'after' if is_future else 'before'

        # Some langs have special pluralization for past and future tense
        try_key_exists = '%s_%s' % (unit, trans_id)
        if try_key_exists != self.translator().transchoice(try_key_exists, count, locale=locale):
            time = self.translator().transchoice(try_key_exists, count, {'count': count}, locale=locale)

        return self.translator().trans(trans_id, {'time': time}, locale=locale)

    # String formatting

    def isoformat(self):
        return self._time.isoformat()

    # Testing aids

    @classmethod
    def set_test_now(cls, test_now=None):
        """
        Set a Time instance (real or mock) to be returned when a "now"
        instance is created.  The provided instance will be returned
        specifically under the following conditions:
            - A call to the classmethod now() method, ex. Time.now()

        To clear the test instance call this method using the default
        parameter of None.

        :type test_now: Date or Pendulum or None
        """
        from .pendulum import Pendulum

        if test_now is not None and not isinstance(test_now, (Pendulum, Time)):
            raise TypeError(
                'Time.set_test_now() only accepts a Time instance, '
                'a Pendulum instance or None.'
            )

        cls._test_now = test_now

    @classmethod
    def get_test_now(cls):
        if cls._test_now is None:
            return None

        if isinstance(cls._test_now, Time):
            return cls._test_now

        return cls._test_now.time()

    # Compatibility methods

    def replace(self, hour=None, minute=None, second=None, microsecond=None,
                tzinfo=True):
        if tzinfo is True:
            tzinfo = self._tzinfo

        hour = hour if hour is not None else self._hour
        minute = minute if minute is not None else self._minute
        second = second if second is not None else self._second
        microsecond = microsecond if microsecond is not None else self._microsecond

        return self.instance(
            self._time.replace(
                hour, minute, second, microsecond,
                tzinfo=tzinfo
            )
        )

    def utcoffset(self):
        return self._time.utcoffset()

    def dst(self):
        return self._time.dst()

    def tzname(self):
        if self._tzinfo is None:
            return None

        return self._time.tzname()

    def _getstate(self, protocol=3):
        tz = self.tzinfo

        return (
            self.hour, self.minute, self.second, self.microsecond,
            tz
        )

    def __reduce__(self):
        return self.__reduce_ex__(2)

    def __reduce_ex__(self, protocol):
        return self.__class__, self._getstate(protocol)

Time.min = Time(0, 0, 0)
Time.max = Time(23, 59, 59, 999999)
Time.resolution = Interval(microseconds=1)



# -*- coding: utf-8 -*-

from __future__ import division

from .parsing import Parser as BaseParser
from .tz import UTC
from .pendulum import Pendulum
from .date import Date
from .time import Time
from ._global import Global


class Parser(BaseParser):
    """
    Parser that returns known types (Pendulum, Date, Time)
    """

    def parse(self, text):
        """
        Parses a string with the given options.

        :param text: The string to parse.
        :type text: str

        :rtype: mixed
        """
        # Handling special cases
        if text == 'now':
            return Pendulum.now()

        parsed = super(Parser, self).parse(text)

        if not self.is_exact():
            return self._create_pendulum_object(parsed)

        # Checking for date
        if 'year' in parsed:
            # Checking for time
            if 'hour' in parsed:
                return self._create_pendulum_object(parsed)
            else:
                return self._create_date_object(parsed)

        return self._create_time_object(parsed)

    def _create_pendulum_object(self, parsed):
        if parsed['offset'] is None:
            tz = self._options.get('tz', UTC)
        else:
            tz = parsed['offset'] / 3600

        return Pendulum(
            parsed['year'], parsed['month'], parsed['day'],
            parsed['hour'], parsed['minute'], parsed['second'],
            parsed['subsecond'],
            tzinfo=tz
        )

    def _create_date_object(self, parsed):
        return Date(
            parsed['year'], parsed['month'], parsed['day']
        )

    def _create_time_object(self, parsed):
        return Time(
            parsed['hour'], parsed['minute'], parsed['second'],
            parsed['subsecond']
        )


def parse(text, **options):
    # Use the mock now value if it exists
    options['now'] = options.get('now', Global.get_test_now())

    return Parser(**options).parse(text)



# -*- coding: utf-8 -*-

translations = {
    # Days
    'days': {
        0: 'söndag',
        1: 'måndag',
        2: 'tisdag',
        3: 'onsdag',
        4: 'torsdag',
        5: 'fredag',
        6: 'lördag'
    },
    'days_abbrev': {
        0: 'sön',
        1: 'mån',
        2: 'tis',
        3: 'ons',
        4: 'tor',
        5: 'fre',
        6: 'lör'
    },

    # Months
    'months': {
        1: 'januari',
        2: 'februari',
        3: 'mars',
        4: 'april',
        5: 'maj',
        6: 'juni',
        7: 'juli',
        8: 'augusti',
        9: 'september',
        10: 'oktober',
        11: 'november',
        12: 'december',
    },
    'months_abbrev': {
        1: 'jan',
        2: 'feb',
        3: 'mar',
        4: 'apr',
        5: 'maj',
        6: 'jun',
        7: 'jul',
        8: 'aug',
        9: 'sep',
        10: 'okt',
        11: 'nov',
        12: 'dec',
    },

    # Units of time
    'year': ['{count} år', '{count} år'],
    'month': ['{count} månad', '{count} månader'],
    'week': ['{count} vecka', '{count} veckor'],
    'day': ['{count} dag', '{count} dagar'],
    'hour': ['{count} timme', '{count} timmar'],
    'minute': ['{count} minut', '{count} minuter'],
    'second': ['{count} sekund', '{count} sekunder'],

    # Relative time
    'ago': '{time} sedan',
    'from_now': 'om {time}',
    'after': '{time} efter',
    'before': '{time} före',

    # Date formats
    'date_formats': {
        'LTS': 'HH:mm:ss',
        'LT': 'HH:mm',
        'LLLL': 'dddd D MMMM YYYY [kl.] HH:mm',
        'LLL': 'D MMMM YYYY [kl.] HH:mm',
        'LL': 'D MMMM YYYY',
        'L': 'YYYY-MM-DD',
    },
}



# -*- coding: utf-8 -*-

translations = {
    # Days
    'days': {
        0: 'კვირა',
        1: 'ორშაბათი',
        2: 'სამშაბათი',
        3: 'ოთხშაბათი',
        4: 'ხუთშაბათი',
        5: 'პარასკევი',
        6: 'შაბათი'
    },
    'days_abbrev': {
        0: 'კვი',
        1: 'ორშ',
        2: 'სამ',
        3: 'ოთხ',
        4: 'ხუთ',
        5: 'პარ',
        6: 'შაბ'
    },

    # Months
    'months': {
        1: 'იანვარი',
        2: 'თებერვალი',
        3: 'მარტი',
        4: 'აპრილი',
        5: 'მაისი',
        6: 'ივნისი',
        7: 'ივლისი',
        8: 'აგვისტო',
        9: 'სექტემბერი',
        10: 'ოქტომბერი',
        11: 'ნოემბერი',
        12: 'დეკემბერი'
    },
    'months_abbrev': {
        1: 'იან',
        2: 'თებ',
        3: 'მარ',
        4: 'აპრ',
        5: 'მაი',
        6: 'ივნ',
        7: 'ივლ',
        8: 'აგვ',
        9: 'სექ',
        10: 'ოქტ',
        11: 'ნოე',
        12: 'დეკ'
    },

    # Units of time
    'year': '{count} წლის',
    'month': '{count} თვის',
    'week': '{count} კვირის',
    'day': '{count} დღის',
    'hour': '{count} საათის',
    'minute': '{count} წუთის',
    'second': '{count} წამის',

    # Relative time
    'ago': '{time} უკან',
    'from_now': '{time} შემდეგ',
    'after': '{time} შემდეგ',
    'before': '{time} უკან',

    # Date formats
    'date_formats': {
        'LTS': 'h:mm:ss A',
        'LT': 'h:mm A',
        'LLLL': 'dddd, D MMMM YYYY h:mm A',
        'LLL': 'D MMMM YYYY h:mm A',
        'LL': 'D MMMM YYYY',
        'L': 'DD/MM/YYYY',
    },
}



# -*- coding: utf-8 -*-

translations = {
    # Days
    'days': {
        0: 'igandea',
        1: 'astelehena',
        2: 'asteartea',
        3: 'asteazkena',
        4: 'osteguna',
        5: 'ostirala',
        6: 'larunbata'
    },
    'days_abbrev': {
        0: 'ig',
        1: 'al',
        2: 'ar',
        3: 'az',
        4: 'og',
        5: 'ol',
        6: 'lr'
    },

    # Months
    'months': {
        1: 'urtarrilak',
        2: 'otsailak',
        3: 'martxoak',
        4: 'apirilak',
        5: 'maiatzak',
        6: 'ekainak',
        7: 'uztailak',
        8: 'abuztuak',
        9: 'irailak',
        10: 'urriak',
        11: 'azaroak',
        12: 'abenduak',
    },
    'months_abbrev': {
        1: 'urt',
        2: 'ots',
        3: 'mar',
        4: 'api',
        5: 'mai',
        6: 'eka',
        7: 'uzt',
        8: 'abu',
        9: 'ira',
        10: 'urr',
        11: 'aza',
        12: 'abe',
    },

    # Units of time
    'year': ['Urte 1', '{count} urte'],
    'month': ['Hile 1', '{count} hile'],
    'week': ['Aste 1', '{count} aste'],
    'day': ['Egun 1', '{count} egun'],
    'hour': ['Ordu 1', '{count} ordu'],
    'minute': ['Minutu 1', '{count} minutu'],
    'second': ['Segundu 1', '{count} segundu'],

    # Relative time
    'ago': 'Orain dela {time}',
    'from_now': '{time} barru',
    'after': '{time} geroago',
    'before': '{time} lehenago',

    # Date formats
    'date_formats': {
        'LTS': 'HH:mm:ss',
        'LT': 'HH:mm',
        'LLLL': 'dddd, YYYY[ko] MMMM[ren] D[a] HH:mm',
        'LLL': 'YYYY[ko] MMMM[ren] D[a] HH:mm',
        'LL': 'YYYY[ko] MMMM[ren] D[a]',
        'L': 'YYYY-MM-DD',
    },
}



# -*- coding: utf-8 -*-

import locale as _locale
from contextlib import contextmanager

from ..translator import Translator
from ..formatting import FORMATTERS


class TranslatableMixin(object):

    _translator = None

    @classmethod
    def translator(cls):
        """
        Initialize the translator instance if necessary.

        :rtype: Translator
        """
        if cls._translator is None:
            cls._translator = Translator('en')
            cls.set_locale('en')

        return cls._translator

    @classmethod
    def set_translator(cls, translator):
        """
        Set the translator instance to use.

        :param translator: The translator
        :type translator: Translator
        """
        cls._translator = translator

    @classmethod
    def get_locale(cls):
        """
        Get the current translator locale.

        :rtype: str
        """
        return cls.translator().locale

    @classmethod
    def set_locale(cls, locale):
        """
        Set the current translator locale and
        indicate if the source locale file exists.

        :type locale: str

        :rtype: bool
        """
        if not cls.translator().has_translations(locale):
            return False

        cls.translator().locale = locale

        return True


class FormattableMixing(object):

    # Default format to use for __str__ method when type juggling occurs.
    DEFAULT_TO_STRING_FORMAT = None

    _to_string_format = DEFAULT_TO_STRING_FORMAT

    _DEFAULT_FORMATTER = 'classic'
    _FORMATTER = _DEFAULT_FORMATTER

    @classmethod
    def reset_to_string_format(cls):
        """
        Reset the format used to the default
        when type juggling a Date instance to a string.
        """
        cls.set_to_string_format(cls.DEFAULT_TO_STRING_FORMAT)

    @classmethod
    def set_to_string_format(cls, fmt):
        """
        Set the default format used
        when type juggling a Date instance to a string

        :type fmt: str
        """
        cls._to_string_format = fmt

    def format(self, fmt, locale=None, formatter=None):
        """
        Formats the instance using the given format.

        :param fmt: The format to use
        :type fmt: str

        :param locale: The locale to use
        :type locale: str or None

        :param formatter: The formatter to use
        :type formatter: str or None

        :rtype: str
        """
        if formatter is None:
            formatter = self._FORMATTER

        if formatter not in FORMATTERS:
            raise ValueError('Invalid formatter [{}]'.format(formatter))

        return FORMATTERS[formatter].format(self, fmt, locale)

    def strftime(self, fmt):
        """
        Formats the instance using the given format.

        :param fmt: The format to use
        :type fmt: str

        :rtype: str
        """
        return self.format(fmt, _locale.getlocale()[0], 'classic')

    @classmethod
    def set_formatter(cls, formatter=None):
        """
        Sets the default string formatter.

        :param formatter: The parameter to set as default.
        :type formatter: str or None
        """
        if formatter is None:
            formatter = cls._DEFAULT_FORMATTER

        if formatter not in FORMATTERS:
            raise ValueError('Invalid formatter [{}]'.format(formatter))

        cls._FORMATTER = formatter

    @classmethod
    def get_formatter(cls):
        """
        Gets the currently used string formatter.

        :rtype: str
        """
        return cls._FORMATTER

    def for_json(self):
        """
        Methods for automatic json serialization by simplejson

        :rtype: str
        """
        return str(self)

    def __format__(self, format_spec):
        if len(format_spec) > 0:
            return self.format(format_spec)

        return str(self)

    def __str__(self):
        if self._to_string_format is None:
            return self.isoformat()

        return self.format(self._to_string_format, formatter='classic')

    def __repr__(self):
        return '<{0} [{1}]>'.format(self.__class__.__name__, str(self))


class TestableMixin(object):

    # A test Pendulum instance to be returned when now instances are created.
    _test_now = None

    @classmethod
    @contextmanager
    def test(cls, mock):
        """
        Context manager to temporarily set the test_now value.

        :type mock: Pendulum or Date or Time or None
        """
        cls.set_test_now(mock)

        yield

        cls.set_test_now()

    @classmethod
    def set_test_now(cls, test_now=None):
        """
        Set a Pendulum instance (real or mock) to be returned when a "now"
        instance is created.  The provided instance will be returned
        specifically under the following conditions:
            - A call to the classmethod now() method, ex. Pendulum.now()
            - When nothing is passed to the constructor or parse(), ex. Pendulum()
            - When the string "now" is passed to parse(), ex. Pendulum.parse('now')

        Note the timezone parameter was left out of the examples above and
        has no affect as the mock value will be returned regardless of its value.

        To clear the test instance call this method using the default
        parameter of null.

        :type test_now: Pendulum or None
        """
        cls._test_now = test_now

    @classmethod
    def get_test_now(cls):
        """
        Get the Pendulum instance (real or mock) to be returned when a "now"
        instance is created.

        :rtype: Pendulum or None
        """
        return cls._test_now

    @classmethod
    def has_test_now(cls):
        return cls._test_now is not None



# -*- coding: utf-8 -*-

translations = {
    # Days
    'days': {
        0: 'կիրակի',
        1: 'երկուշաբթի',
        2: 'երեքշաբթի',
        3: 'չորեքշաբթի',
        4: 'հինգշաբթի',
        5: 'ուրբաթ',
        6: 'շաբաթ'
    },
    'days_abbrev': {
        0: 'կրկ',
        1: 'երկ',
        2: 'երք',
        3: 'չրք',
        4: 'հնգ',
        5: 'ուրբ',
        6: 'շբթ'
    },

    # Months
    'months': {
        1: 'հունվար',
        2: 'փետրվար',
        3: 'մարտ',
        4: 'ապրիլ',
        5: 'մայիս',
        6: 'հունիս',
        7: 'հուլիս',
        8: 'օգոստոս',
        9: 'սեպտեմբեր',
        10: 'հոկտեմբեր',
        11: 'նոյեմբեր',
        12: 'դեկտեմբեր',
    },
    'months_abbrev': {
        1: 'հնվ',
        2: 'փտր',
        3: 'մրտ',
        4: 'ապր',
        5: 'մյս',
        6: 'հնս',
        7: 'հլս',
        8: 'օգս',
        9: 'սպտ',
        10: 'հկտ',
        11: 'նմբ',
        12: 'դկտ',
    },

    # Units of time
    'year': '{count} տարի',
    'month': '{count} ամիս',
    'week': '{count} շաբաթ',
    'day': '{count} օր',
    'hour': '{count} ժամ',
    'minute': '{count} րոպե',
    'second': '{count} վայրկյան',

    # Relative time
    'ago': '{time} առաջ',
    'from_now': '{time} հետո',
    'after': '{time} հետո',
    'before': '{time} առաջ',

    # Date formats
    'date_formats': {
        'LTS': 'HH:mm:ss',
        'LT': 'HH:mm',
        'LLLL': 'dddd, D MMMM YYYY թ., HH:mm',
        'LLL': 'D MMMM YYYY թ., HH:mm',
        'LL': 'D MMMM YYYY թ.',
        'L': 'DD.MM.YYYY',
    },
}



# -*- coding: utf-8 -*-

from .parser import Parser


def parse(text, **options):
    """
    Parses a string with the given options.

    :param text: The string to parse.
    :type text: str

    :param options: The parsing options.
    :type options: dict

    :rtype: dict

    :raises: ParserError
    """
    return Parser(**options).parse(text)



# -*- coding: utf-8 -*-

import re
import datetime

from .._compat import decode
from .formatter import Formatter


class ClassicFormatter(Formatter):

    _CUSTOM_FORMATTERS = ['_z', '_t']
    _FORMATTERS_REGEX = re.compile('%%(%s)' % '|'.join(_CUSTOM_FORMATTERS))

    def format(self, dt, fmt, locale=None):
        """
        Formats a Pendulum instance with a given format and locale.

        :param dt: The instance to format
        :type dt: pendulum.Pendulum or pendulum.Date

        :param fmt: The format to use
        :type fmt: str

        :param locale: The locale to use
        :type locale: str or None

        :rtype: str
        """
        if not locale:
            locale = dt.get_locale()

        # Checking for custom formatters
        fmt = self._FORMATTERS_REGEX.sub(lambda m: self._strftime(dt, m, locale), fmt)

        # Checking for localizable directives
        fmt = re.sub('%(a|A|b|B|p)', lambda m: self._localize_directive(dt, m.group(1), locale), fmt)

        if hasattr(dt, '_datetime'):
            trans = dt._datetime.strftime(fmt)
        elif hasattr(dt, '_time'):
            trans = dt._time.strftime(fmt)
        else:
            trans = datetime.date(dt.year, dt.month, dt.day).strftime(fmt)

        return decode(trans)

    def _localize_directive(self, dt, directive, locale):
        """
        Localize a native strftime directive.

        :param dt: The instance to format
        :type dt: pendulum.Pendulum

        :param directive: The directive to localize
        :type directive: str

        :param locale: The locale to use for localization
        :type locale: str

        :rtype: str
        """
        if directive == 'a':
            id = 'days_abbrev'
            number = dt.day_of_week
        elif directive == 'A':
            id = 'days'
            number = dt.day_of_week
        elif directive == 'b':
            id = 'months_abbrev'
            number = dt.month
        elif directive == 'B':
            id = 'months'
            number = dt.month
        elif directive == 'p':
            id = 'meridian'
            number = (dt.hour, dt.minute)
        else:
            raise ValueError('Unlocalizable directive [{}]'.format(directive))

        translation = dt.translator().transchoice(id, number, locale=locale)
        if translation == id:
            return ''

        return translation

    def _strftime(self, dt, m, locale):
        """
        Handles custom formatters in format string.

        :param dt: The instance to format
        :type dt: pendulum.Pendulum

        :return: str
        """
        fmt = m.group(1)

        if fmt == '_z':
            offset = dt.utcoffset() or datetime.timedelta()
            minutes = offset.total_seconds() / 60

            if minutes >= 0:
                sign = '+'
            else:
                sign = '-'

            hour, minute = divmod(abs(int(minutes)), 60)

            return '{0}{1:02d}:{2:02d}'.format(sign, hour, minute)
        elif fmt == '_t':
            translation = dt.translator().transchoice('ordinal', dt.day, locale=locale)
            if translation == 'ordinal':
                translation = ''

            return translation

        raise ValueError('Unknown formatter %%{}'.format(fmt))



# -*- coding: utf-8 -*-

from .._compat import decode
from ..translator import Translator


class DifferenceFormatter(object):
    """
    Handles formatting differences in text.
    """

    def __init__(self, translator=Translator()):
        self._translator = translator

    def diff_for_humans(self, date, other=None, absolute=False, locale=None):
        """
        Get the difference in a human readable format.

        :param date: The datetime to start with.
        :type date: pendulum.Date or pendulum.Pendulum

        :param other: The datetime to compare against (defaults to now).
        :type other: pendulum.Date or pendulum.Pendulum or None

        :param absolute: Removes time difference modifiers ago, after, etc
        :type absolute: bool

        :param locale: The locale to use
        :type locale: str or None

        :rtype: str
        """
        is_now = other is None

        if is_now:
            if hasattr(date, 'now'):
                other = date.now(date.timezone)
            else:
                other = date.today()

        diff = date.diff(other)

        count = diff.remaining_seconds
        unit = 'second'

        if diff.years > 0:
            unit = 'year'
            count = diff.years

            if diff.months > 6:
                count += 1
        elif diff.months == 11 and (diff.weeks * 7 + diff.remaining_days) > 15:
            unit = 'year'
            count = 1
        elif diff.months > 0:
            unit = 'month'
            count = diff.months

            if (diff.weeks * 7 + diff.remaining_days) >= 27:
                count += 1
        elif diff.weeks > 0:
            unit = 'week'
            count = diff.weeks

            if diff.remaining_days > 3:
                count += 1
        elif diff.remaining_days > 0:
            unit = 'day'
            count = diff.remaining_days
        elif diff.hours > 0:
            unit = 'hour'
            count = diff.hours
        elif diff.minutes > 0:
            unit = 'minute'
            count = diff.minutes

        if count == 0:
            count = 1

        time = self._translator.transchoice(unit, count, {'count': count}, locale=locale)

        if absolute:
            return decode(time)

        is_future = diff.invert

        if is_now:
            trans_id = 'from_now' if is_future else 'ago'
        else:
            trans_id = 'after' if is_future else 'before'

        # Some langs have special pluralization for past and future tense
        try_key_exists = '%s_%s' % (unit, trans_id)
        if try_key_exists != self._translator.transchoice(try_key_exists, count, locale=locale):
            time = self._translator.transchoice(try_key_exists, count, {'count': count}, locale=locale)

        trans = self._translator.trans(trans_id, {'time': time}, locale=locale)

        return decode(trans)



# -*- coding: utf-8 -*-

translations = {
    # Days
    'days': {
        0: 'domingo',
        1: 'lunes',
        2: 'martes',
        3: 'miércoles',
        4: 'jueves',
        5: 'viernes',
        6: 'sábado',
    },
    'days_abbrev': {
        0: 'dom',
        1: 'lun',
        2: 'mar',
        3: 'mie',
        4: 'jue',
        5: 'vie',
        6: 'sab'
    },

    # Months
    'months': {
        1: 'enero',
        2: 'febrero',
        3: 'marzo',
        4: 'abril',
        5: 'mayo',
        6: 'junio',
        7: 'julio',
        8: 'agosto',
        9: 'septiembre',
        10: 'octubre',
        11: 'noviembre',
        12: 'diciembre',
    },
    'months_abbrev': {
        1: 'ene',
        2: 'feb',
        3: 'mar',
        4: 'abr',
        5: 'may',
        6: 'jun',
        7: 'jul',
        8: 'ago',
        9: 'sep',
        10: 'oct',
        11: 'nov',
        12: 'dec',
    },

    # Units of time
    'year': ['{count} año', '{count} años'],
    'month': ['{count} mes', '{count} meses'],
    'week': ['{count} semana', '{count} semanas'],
    'day': ['{count} día', '{count} días'],
    'hour': ['{count} hora', '{count} horas'],
    'minute': ['{count} minuto', '{count} minutos'],
    'second': ['{count} segundo', '{count} segundos'],

    # Relative time
    'ago': 'hace {time}',
    'from_now': 'dentro de {time}',
    'after': '{time} después',
    'before': '{time} antes',

    # Ordinals
    'ordinal': 'º',

    # Date formats
    'date_formats': {
        'LTS': 'H:mm:ss',
        'LT': 'H:mm',
        'LLLL': 'dddd, D [de] MMMM [de] YYYY H:mm',
        'LLL': 'D [de] MMMM [de] YYYY H:mm',
        'LL': 'D [de] MMMM [de] YYYY',
        'L': 'DD/MM/YYYY',
    },
}



# -*- coding: utf-8 -*-

translations = {
    # Days
    'days': {
        0: ' Недела',
        1: 'Понеделник',
        2: ' Вторник',
        3: ' Среда',
        4: ' Четврток',
        5: ' Петок',
        6: ' Сабота'
    },
    'days_abbrev': {
        0: ' Нед.',
        1: 'Пон.',
        2: ' Вт.',
        3: ' Сре.',
        4: ' Чет.',
        5: ' Пет.',
        6: ' Саб.'
    },

    # Months
    'months': {
        1: 'Јануари',
        2: 'Февруари',
        3: 'Март',
        4: 'Април',
        5: 'Мај',
        6: 'Јуни',
        7: 'Јули',
        8: 'Август',
        9: 'Септември',
        10: 'Октомври',
        11: 'Ноември',
        12: 'Декември',
    },
    'months_abbrev': {
        1: 'Јан.',
        2: ' Фев.',
        3: ' Мар.',
        4: ' Апр.',
        5: ' Мај',
        6: ' Јун.',
        7: ' Јул.',
        8: ' Авг.',
        9: ' Септ.',
        10: ' Окт.',
        11: ' Ноем.',
        12: ' Декем.',
    },

    # Units of time
    'year': ['{count} година', '{count} години'],
    'month': ['{count} месец', '{count} месеци'],
    'week': ['{count} седмица', '{count} седмици'],
    'day': ['{count} ден', '{count} дена'],
    'hour': ['{count} час', '{count} часа'],
    'minute': ['{count} минута', '{count} минути'],
    'second': ['{count} секунда', '{count} секунди'],

    # Relative time
    'ago': 'пред {time}',
    'from_now': '{time} од сега',
    'after': 'по {time}',
    'before': 'пред {time}',

    # Date formats
    'date_formats': {
        'LTS': 'H:mm:ss',
        'LT': 'H:mm',
        'LLLL': 'dddd, D MMMM YYYY H:mm',
        'LLL': 'D MMMM YYYY H:mm',
        'LL': 'D MMMM YYYY',
        'L': 'D.MM.YYYY',
    },
}



# -*- coding: utf-8 -*-

translations = {
    # Days
    'days': {
        0: 'Sunday',
        1: 'Monday',
        2: 'Tuesday',
        3: 'Wednesday',
        4: 'Thursday',
        5: 'Friday',
        6: 'Saturday'
    },
    'days_abbrev': {
        0: 'Sun',
        1: 'Mon',
        2: 'Tue',
        3: 'Wed',
        4: 'Thu',
        5: 'Fri',
        6: 'Sat'
    },

    # Months
    'months': {
        1: 'January',
        2: 'February',
        3: 'March',
        4: 'April',
        5: 'May',
        6: 'June',
        7: 'July',
        8: 'August',
        9: 'September',
        10: 'October',
        11: 'November',
        12: 'December',
    },
    'months_abbrev': {
        1: 'Jan',
        2: 'Feb',
        3: 'Mar',
        4: 'Apr',
        5: 'May',
        6: 'Jun',
        7: 'Jul',
        8: 'Aug',
        9: 'Sep',
        10: 'Oct',
        11: 'Nov',
        12: 'Dec',
    },

    # Units of time
    'year': ['{count} year', '{count} years'],
    'month': ['{count} month', '{count} months'],
    'week': ['{count} week', '{count} weeks'],
    'day': ['{count} day', '{count} days'],
    'hour': ['{count} hour', '{count} hours'],
    'minute': ['{count} minute', '{count} minutes'],
    'second': ['{count} second', '{count} seconds'],

    # Relative time
    'ago': '{time} ago',
    'from_now': '{time} from now',
    'after': '{time} after',
    'before': '{time} before',

    # Ordinals
    'ordinal': lambda x: 'th' if 10 <= x % 100 < 20 else {1: 'st', 2: 'nd', 3: 'rd'}.get(x % 10, "th"),

    # Meridians
    # [time] is a (hour, minute) tuple
    'meridian': lambda time: 'AM' if 0 <= time[0] < 12 else 'PM',

    # Date formats
    'date_formats': {
        'LTS': 'h:mm:ss A',
        'LT': 'h:mm A',
        'L': 'MM/DD/YYYY',
        'LL': 'MMMM D, YYYY',
        'LLL': 'MMMM D, YYYY h:mm A',
        'LLLL': 'dddd, MMMM D, YYYY h:mm A',
    },
}



# -*- coding: utf-8 -*-

import pytzdata

from .timezone import Timezone, FixedTimezone, UTC
from .local_timezone import LocalTimezone


timezones = pytzdata.timezones


def timezone(name):
    """
    Loads a Timezone instance by name.

    :param name: The name of the timezone.
    :type name: str or int

    :rtype: Timezone
    """
    return Timezone.load(name)


def local_timezone():
    """
    Loads the local timezone.

    :rtype: Timezone
    """
    return LocalTimezone.get()



# -*- coding: utf-8 -*-

import sys

PY2 = sys.version_info[0] == 2
PY3K = sys.version_info[0] >= 3
PY36 = sys.version_info >= (3, 6)


try:               # Python 2
    long
    unicode
    basestring
except NameError:  # Python 3
    long = int
    unicode = str
    basestring = str

try:               # Python >= 3.3
    FileNotFoundError = FileNotFoundError
except NameError:  # Python < 3.3
    FileNotFoundError = IOError  # cf PEP-3151 


def decode(string, encodings=None):
    if not PY2 and not isinstance(string, bytes):
        return string

    if PY2 and isinstance(string, unicode):
        return string

    encodings = encodings or ['utf-8', 'latin1', 'ascii']

    for encoding in encodings:
        try:
            return string.decode(encoding)
        except (UnicodeEncodeError, UnicodeDecodeError):
            pass

    return string.decode(encodings[0], errors='ignore')


def encode(string, encodings=None):
    if not PY2 and isinstance(string, bytes):
        return string

    if PY2 and isinstance(string, str):
        return string

    encodings = encodings or ['utf-8', 'latin1', 'ascii']

    for encoding in encodings:
        try:
            return string.encode(encoding)
        except (UnicodeEncodeError, UnicodeDecodeError):
            pass

    return string.encode(encodings[0], errors='ignore')



# -*- coding: utf-8 -*-

translations = {
    # Days
    'days': {
        0: 'zondag',
        1: 'maandag',
        2: 'dinsdag',
        3: 'woensdag',
        4: 'donderdag',
        5: 'vrijdag',
        6: 'zaterdag'
    },
    'days_abbrev': {
        0: 'zo',
        1: 'ma',
        2: 'di',
        3: 'wo',
        4: 'do',
        5: 'vr',
        6: 'za'
    },

    # Months
    'months': {
        1: 'januari',
        2: 'februari',
        3: 'maart',
        4: 'april',
        5: 'mei',
        6: 'juni',
        7: 'juli',
        8: 'augustus',
        9: 'september',
        10: 'oktober',
        11: 'november',
        12: 'december',
    },
    'months_abbrev': {
        1: 'jan',
        2: 'feb',
        3: 'mrt',
        4: 'apr',
        5: 'mei',
        6: 'jun',
        7: 'jul',
        8: 'aug',
        9: 'sep',
        10: 'okt',
        11: 'nov',
        12: 'dec',
    },

    # Units of time
    'year': '{count} jaar',
    'month': ['{count} maand', '{count} maanden'],
    'week': ['{count} week', '{count} weken'],
    'day': ['{count} dag', '{count} dagen'],
    'hour': '{count} uur',
    'minute': ['{count} minuut', '{count} minuten'],
    'second': ['{count} seconde', '{count} seconden'],

    # Relative time
    'ago': '{time} geleden',
    'from_now': 'over {time}',
    'after': '{time} later',
    'before': '{time} eerder',

    # Date formats
    'date_formats': {
        'LTS': 'HH:mm:ss',
        'LT': 'HH:mm',
        'LLLL': 'dddd D MMMM YYYY HH:mm',
        'LLL': 'D MMMM YYYY HH:mm',
        'LL': 'D MMMM YYYY',
        'L': 'DD-MM-YYYY',
    },
}



# -*- coding: utf-8 -*-

translations = {
    # Days
    'days': {
        0: 'อาทิตย์',
        1: 'จันทร์',
        2: 'อังคาร',
        3: 'พุธ',
        4: 'พฤหัสบดี',
        5: 'ศุกร์',
        6: 'เสาร์'
    },
    'days_abbrev': {
        0: 'อา',
        1: 'จ',
        2: 'อ',
        3: 'พ',
        4: 'พฤ',
        5: 'ศ',
        6: 'ส'
    },

    # Months
    'months': {
        1: 'มกราคม',
        2: 'กุมภาพันธ์',
        3: 'มีนาคม',
        4: 'เมษายน',
        5: 'พฤษภาคม',
        6: 'มิถุนายน',
        7: 'กรกฎาคม',
        8: 'สิงหาคม',
        9: 'กันยายน',
        10: 'ตุลาคม',
        11: 'พฤศจิกายน',
        12: 'ธันวาคม',
    },
    'months_abbrev': {
        1: 'ม.ค.',
        2: 'ก.พ.',
        3: 'มี.ค.',
        4: 'เม.ย.',
        5: 'พ.ค.',
        6: 'มิ.ย.',
        7: 'ก.ค.',
        8: 'ส.ค.',
        9: 'ก.ย.',
        10: 'ต.ค.',
        11: 'พ.ย.',
        12: 'ธ.ค.',
    },

    # Units of time
    'year': ['{count} ปี', '{count} ปี'],
    'month': ['{count} เดือน', '{count} เดือน'],
    'week': ['{count} สัปดาห์', '{count} สัปดาห์'],
    'day': ['{count} วัน', '{count} วัน'],
    'hour': ['{count} ชั่วโมง', '{count} ชั่วโมง'],
    'minute': ['{count} นาที', '{count} นาที'],
    'second': ['{count} วินาที', '{count} วินาที'],

    # Relative time
    'ago': '{time} ที่แล้ว',
    'from_now': '{time} จากนี้',
    'after': 'หลัง{time}',
    'before': 'ก่อน{time}',

    # Date formats
    'date_formats': {
        'LTS': 'H นาฬิกา m นาที s วินาที',
        'LT': 'H นาฬิกา m นาที',
        'LLLL': 'วันddddที่ D MMMM YYYY เวลา H นาฬิกา m นาที',
        'LLL': 'D MMMM YYYY เวลา H นาฬิกา m นาที',
        'LL': 'D MMMM YYYY',
        'L': 'YYYY/MM/DD',
    },
}



# -*- coding: utf-8 -*-

translations = {
    # Days
    'days': {
        0: 'vasárnap',
        1: 'hétfő',
        2: 'kedd',
        3: 'szerda',
        4: 'csütörtök',
        5: 'péntek',
        6: 'szombat'
    },
    'days_abbrev': {
        0: 'vas',
        1: 'hét',
        2: 'kedd',
        3: 'szer',
        4: 'csüt',
        5: 'pént',
        6: 'szom'
    },

    # Months
    'months': {
        1: 'január',
        2: 'február',
        3: 'március',
        4: 'április',
        5: 'május',
        6: 'június',
        7: 'július',
        8: 'augusztus',
        9: 'szeptember',
        10: 'október',
        11: 'november',
        12: 'december',
    },
    'months_abbrev': {
        1: 'jan',
        2: 'febr',
        3: 'márc',
        4: 'ápr',
        5: 'máj',
        6: 'jún',
        7: 'júl',
        8: 'aug',
        9: 'szept',
        10: 'okt',
        11: 'nov',
        12: 'dec',
    },

    # Units of time
    'year': '{count} év',
    'month': '{count} hónap',
    'week': '{count} hét',
    'day': '{count} nap',
    'hour': '{count} óra',
    'minute': '{count} perc',
    'second': '{count} másodperc',

    # Relative time
    'ago': '{time}',
    'from_now': '{time} múlva',
    'after': '{time} később',
    'before': '{time} korábban',

    'year_ago': '{count} éve',
    'month_ago': '{count} hónapja',
    'week_ago': '{count} hete',
    'day_ago': '{count} napja',
    'hour_ago': '{count} órája',
    'minute_ago': '{count} perce',
    'second_ago': '{count} másodperce',

    'year_after': '{count} évvel',
    'month_after': '{count} hónappal',
    'week_after': '{count} héttel',
    'day_after': '{count} nappal',
    'hour_after': '{count} órával',
    'minute_after': '{count} perccel',
    'second_after': '{count} másodperccel',

    'year_before': '{count} évvel',
    'month_before': '{count} hónappal',
    'week_before': '{count} héttel',
    'day_before': '{count} nappal',
    'hour_before': '{count} órával',
    'minute_before': '{count} perccel',
    'second_before': '{count} másodperccel',

    # Meridians
    'meridian': lambda time: 'DE' if 0 <= time[0] < 12 else 'DU',

    # Date formats
    'date_formats': {
        'LTS': 'H:mm:ss',
        'LT': 'H:mm',
        'LLLL': 'YYYY. MMMM D., dddd H:mm',
        'LLL': 'YYYY. MMMM D. H:mm',
        'LL': 'YYYY. MMMM D.',
        'L': 'YYYY.MM.DD.',
    },
}



# -*- coding: utf-8 -*-

translations = {
    # Days
    'days': {
        0: 'Domingo',
        1: 'Luns',
        2: 'Martes',
        3: 'Mércores',
        4: 'Xoves',
        5: 'Venres',
        6: 'Sábado'
    },
    'days_abbrev': {
        0: 'Dom.',
        1: 'Lun.',
        2: 'Mar.',
        3: 'Mér.',
        4: 'Xov.',
        5: 'Ven.',
        6: 'Sáb.'
    },

    # Months
    'months': {
        1: 'Xaneiro',
        2: 'Febreiro',
        3: 'Marzo',
        4: 'Abril',
        5: 'Maio',
        6: 'Xuño',
        7: 'Xullo',
        8: 'Agosto',
        9: 'Setembro',
        10: 'Outubro',
        11: 'Novembro',
        12: 'Decembro',
    },
    'months_abbrev': {
        1: 'Xan.',
        2: 'Feb.',
        3: 'Mar.',
        4: 'Abr.',
        5: 'Mai.',
        6: 'Xuñ.',
        7: 'Xul.',
        8: 'Ago.',
        9: 'Set.',
        10: 'Out.',
        11: 'Nov.',
        12: 'Dec.'
    },

    # Units of time
    'year': ['{count} ano', '{count} anos'],
    'month': ['{count} mes', '{count} meses'],
    'week': ['{count} semana', '{count} semanas'],
    'day': ['{count} día', '{count} días'],
    'hour': ['{count} hora', '{count} horas'],
    'minute': ['{count} minuto', '{count} minutos'],
    'second': ['{count} segundo', '{count} segundos'],

    # Relative time
    'ago': 'fai {time}',
    'from_now': 'dentro de {time}',
    'after': '{time} despois',
    'before': '{time} antes',

    # Ordinals
    'ordinal': 'º',

    # Date formats
    'date_formats': {
        'LTS': 'H:mm:ss',
        'LT': 'H:mm',
        'LLLL': 'dddd D MMMM YYYY H:mm',
        'LLL': 'D MMMM YYYY H:mm',
        'LL': 'D MMMM YYYY',
        'L': 'DD/MM/YYYY',
    },
}



# -*- coding: utf-8 -*-


class Formatter(object):
    """
    Base class for all formatters.
    """

    def format(self, dt, fmt, locale=None):
        """
        Formats a Pendulum instance with a given format and locale.

        :param dt: The instance to format
        :type dt: Pendulum

        :param fmt: The format to use
        :type fmt: str

        :param locale: The locale to use
        :type locale: str or None

        :rtype: str
        """
        raise NotImplementedError()



# -*- coding: utf-8 -*-

import re
from .lang import TRANSLATIONS


class Translator(object):

    def __init__(self, locale='en'):
        self._locale = self._format_locale(locale)
        self._translations = TRANSLATIONS

    @property
    def locale(self):
        return self._locale

    @locale.setter
    def locale(self, locale):
        self._locale = locale

    def trans(self, id, parameters=None, locale=None):
        if parameters is None:
            parameters = {}

        if not locale:
            locale = self._locale
        else:
            locale = self._format_locale(locale)

        while not self.has_translations(locale):
            fallback = locale.split('_')[0]
            if locale == fallback:
                raise ValueError('Locale [{}] could not be found.'.format(locale))

        if id not in self._translations[locale]:
            return id

        translation = self._translations[locale][id]

        return translation.format(**parameters)

    def transchoice(self, id, number, parameters=None, locale=None):
        if parameters is None:
            parameters = {}

        if 'count' not in parameters:
            parameters['count'] = number

        if not locale:
            locale = self._locale
        else:
            locale = self._format_locale(locale)

        while not self.has_translations(locale):
            fallback = locale.split('_')[0]
            if locale == fallback:
                raise ValueError('Locale [{}] could not be found.'.format(locale))

            locale = fallback

        if id not in self._translations[locale]:
            return id

        translation = self._translations[locale][id]

        if isinstance(translation, list):
            translation = translation[PluralizationRules.get(number, locale)]
        elif isinstance(translation, dict):
            if number in translation:
                translation = translation[number]
            else:
                for key in translation.keys():
                    if isinstance(key, tuple) and number in range(*key):
                        return translation[key].format(**parameters)

                translation = translation['default']
        elif callable(translation):
            translation = translation(number)

        return translation.format(**parameters)

    def has_translations(self, locale):
        locale = self._format_locale(locale)

        return locale in self._translations

    def add_translations(self, locale, translations):
        self._translations[locale] = translations

    def _format_locale(cls, locale):
        """
        Properly format locale.

        :param locale: The locale
        :type locale: str

        :rtype: str
        """
        m = re.match('([a-z]{2})[-_]([a-z]{2})', locale, re.I)
        if m:
            return '{}_{}'.format(m.group(1).lower(), m.group(2).lower())
        else:
            return locale.lower()


class PluralizationRules(object):
    """
    Returns the plural rules for a given locale.

    Attributes:
        rules  list
    """

    # The plural rules are derived from code of the Zend Framework (2010-09-25),
    # which is subject to the new BSD license (http://framework.zend.com/license/new-bsd).
    # Copyright (c) 2005-2010 Zend Technologies USA Inc. (http://www.zend.com)
    _rules = {
        'bo': lambda number: 0,
        'dz': lambda number: 0,
        'id': lambda number: 0,
        'ja': lambda number: 0,
        'jv': lambda number: 0,
        'ka': lambda number: 0,
        'km': lambda number: 0,
        'kn': lambda number: 0,
        'ko': lambda number: 0,
        'ms': lambda number: 0,
        'th': lambda number: 0,
        'tr': lambda number: 0,
        'vi': lambda number: 0,
        'zh': lambda number: 0,
        'af': lambda number: 0 if number == 1 else 1,
        'az': lambda number: 0 if number == 1 else 1,
        'bn': lambda number: 0 if number == 1 else 1,
        'bg': lambda number: 0 if number == 1 else 1,
        'ca': lambda number: 0 if number == 1 else 1,
        'da': lambda number: 0 if number == 1 else 1,
        'de': lambda number: 0 if number == 1 else 1,
        'el': lambda number: 0 if number == 1 else 1,
        'en': lambda number: 0 if number == 1 else 1,
        'eo': lambda number: 0 if number == 1 else 1,
        'es': lambda number: 0 if number == 1 else 1,
        'et': lambda number: 0 if number == 1 else 1,
        'eu': lambda number: 0 if number == 1 else 1,
        'fa': lambda number: 0 if number == 1 else 1,
        'fi': lambda number: 0 if number == 1 else 1,
        'fo': lambda number: 0 if number == 1 else 1,
        'fur': lambda number: 0 if number == 1 else 1,
        'fy': lambda number: 0 if number == 1 else 1,
        'gl': lambda number: 0 if number == 1 else 1,
        'gu': lambda number: 0 if number == 1 else 1,
        'ha': lambda number: 0 if number == 1 else 1,
        'he': lambda number: 0 if number == 1 else 1,
        'hu': lambda number: 0 if number == 1 else 1,
        'is': lambda number: 0 if number == 1 else 1,
        'it': lambda number: 0 if number == 1 else 1,
        'ku': lambda number: 0 if number == 1 else 1,
        'lb': lambda number: 0 if number == 1 else 1,
        'ml': lambda number: 0 if number == 1 else 1,
        'mn': lambda number: 0 if number == 1 else 1,
        'mr': lambda number: 0 if number == 1 else 1,
        'nah': lambda number: 0 if number == 1 else 1,
        'nb': lambda number: 0 if number == 1 else 1,
        'ne': lambda number: 0 if number == 1 else 1,
        'nl': lambda number: 0 if number == 1 else 1,
        'nn': lambda number: 0 if number == 1 else 1,
        'no': lambda number: 0 if number == 1 else 1,
        'om': lambda number: 0 if number == 1 else 1,
        'or': lambda number: 0 if number == 1 else 1,
        'pa': lambda number: 0 if number == 1 else 1,
        'pap': lambda number: 0 if number == 1 else 1,
        'ps': lambda number: 0 if number == 1 else 1,
        'pt': lambda number: 0 if number == 1 else 1,
        'so': lambda number: 0 if number == 1 else 1,
        'sq': lambda number: 0 if number == 1 else 1,
        'sv': lambda number: 0 if number == 1 else 1,
        'sw': lambda number: 0 if number == 1 else 1,
        'ta': lambda number: 0 if number == 1 else 1,
        'te': lambda number: 0 if number == 1 else 1,
        'tk': lambda number: 0 if number == 1 else 1,
        'ur': lambda number: 0 if number == 1 else 1,
        'zu': lambda number: 0 if number == 1 else 1,
        'am': lambda number: 0 if number in (0, 1) else 1,
        'bh': lambda number: 0 if number in (0, 1) else 1,
        'fil': lambda number: 0 if number in (0, 1) else 1,
        'fr': lambda number: 0 if number in (0, 1) else 1,
        'gun': lambda number: 0 if number in (0, 1) else 1,
        'hi': lambda number: 0 if number in (0, 1) else 1,
        'ln': lambda number: 0 if number in (0, 1) else 1,
        'mg': lambda number: 0 if number in (0, 1) else 1,
        'nso': lambda number: 0 if number in (0, 1) else 1,
        'xbr': lambda number: 0 if number in (0, 1) else 1,
        'ti': lambda number: 0 if number in (0, 1) else 1,
        'wa': lambda number: 0 if number in (0, 1) else 1,
        'be': lambda number: 0 if number % 10 == 1 and number % 100 != 11 else (1 if ((number % 10 >= 2) and (number % 10 <= 4)) and ((number % 100 < 10) or (number % 100 >= 20)) else 2),
        'bs': lambda number: 0 if number % 10 == 1 and number % 100 != 11 else (1 if ((number % 10 >= 2) and (number % 10 <= 4)) and ((number % 100 < 10) or (number % 100 >= 20)) else 2),
        'hr': lambda number: 0 if number % 10 == 1 and number % 100 != 11 else (1 if ((number % 10 >= 2) and (number % 10 <= 4)) and ((number % 100 < 10) or (number % 100 >= 20)) else 2),
        'ru': lambda number: 0 if number % 10 == 1 and number % 100 != 11 else (1 if ((number % 10 >= 2) and (number % 10 <= 4)) and ((number % 100 < 10) or (number % 100 >= 20)) else 2),
        'sr': lambda number: 0 if number % 10 == 1 and number % 100 != 11 else (1 if ((number % 10 >= 2) and (number % 10 <= 4)) and ((number % 100 < 10) or (number % 100 >= 20)) else 2),
        'uk': lambda number: 0 if number % 10 == 1 and number % 100 != 11 else (1 if ((number % 10 >= 2) and (number % 10 <= 4)) and ((number % 100 < 10) or (number % 100 >= 20)) else 2),
        'cs': lambda number: 0 if number == 1 else (1 if 2 <= number <= 4 else 2),
        'sk': lambda number: 0 if number == 1 else (1 if 2 <= number <= 4 else 2),
        'ga': lambda number: 0 if number == 1 else (1 if number == 2 else 2),
        'lt': lambda number: 0 if (number % 10 == 1 and number % 100 != 11) else (1 if ((number % 10 >= 2 and number % 100 < 10) or number % 100 >= 20) else 2),
        'sl': lambda number: 0 if number % 100 == 1 else (1 if number % 100 == 2 else (2 if number % 100 in (3, 4) else 3)),
        'mk': lambda number: 0 if number % 10 == 1 else 1,
        'mt': lambda number: 0 if number == 1 else (1 if number == 0 or 1 < number % 100 < 11 else (2 if 10 < number % 100 < 20 else 3)),
        'lv': lambda number: 0 if number == 0 else (1 if number % 10 != 1 and number % 100 != 11 else 2),
        'pl': lambda number: 0 if number == 1 else (1 if (2 <= number % 10 <= 4) and (number % 100 < 12 or number % 100 > 14) else 2),
        'cy': lambda number: 0 if number == 1 else (1 if number == 2 else (2 if number in (8, 11) else 3)),
        'ro': lambda number: 0 if number == 1 else (1 if number == 0 or 0 < number % 100 < 20 else 2),
        'ar': lambda number: 0 if number == 0 else (1 if number == 1 else (2 if number == 2 else (3 if 3 <= number % 100 <= 10 else (4 if 11 <= number % 100 <= 99 else 5))))
    }

    @staticmethod
    def get(number, locale):
        """
        Returns the plural position to use for the given locale and number.

        @type number: int
        @param number: The number

        @type locale: str
        @param locale: The locale

        @rtype: int
        @return: The plural position
        """
        if locale == 'pt_br':
            # temporary set a locale for brazilian
            locale = 'xbr'

        if len(locale) > 3:
            locale = locale.split("_")[0]

        rule = PluralizationRules._rules.get(locale, lambda _: 0)
        _return = rule(number)
        if not isinstance(_return, int) or _return < 0:
            return 0
        return _return

    @staticmethod
    def set(rule, locale):
        """
        Overrides the default plural rule for a given locale.

        @type rule: str
        @param rule: Callable

        @type locale: str
        @param locale: The locale

        @raises: ValueError
        """

        if locale == 'pt_br':
            # temporary set a locale for brazilian
            locale = 'xbr'

        if len(locale) > 3:
            locale = locale.split("_")[0]

        if not hasattr(rule, '__call__'):
            raise ValueError('The given rule can not be called')

        PluralizationRules._rules[locale] = rule



# -*- coding: utf-8 -*-

translations = {
    # Days
    'days': {
        0: 'ראשון',
        1: 'שני',
        2: 'שלישי',
        3: 'רביעי',
        4: 'חמישי',
        5: 'שישי',
        6: 'שבת'
    },
    'days_abbrev': {
        0: 'א׳',
        1: 'ב׳',
        2: 'ג׳',
        3: 'ד׳',
        4: 'ה׳',
        5: 'ו׳',
        6: 'ש׳'
    },

    # Months
    'months': {
        1: 'ינואר',
        2: 'פברואר',
        3: 'מרץ',
        4: 'אפריל',
        5: 'מאי',
        6: 'יוני',
        7: 'יולי',
        8: 'אוגוסט',
        9: 'ספטמבר',
        10: 'אוקטובר',
        11: 'נובמבר',
        12: 'דצמבר',
    },
    'months_abbrev': {
        1: 'ינו׳',
        2: 'פבר׳',
        3: 'מרץ',
        4: 'אפר׳',
        5: 'מאי',
        6: 'יוני',
        7: 'יולי',
        8: 'אוג׳',
        9: 'ספט׳',
        10: 'אוק׳',
        11: 'נוב׳',
        12: 'דצמ׳',
    },

    # Units of time
    'year': {
        1: 'שנה',
        2: 'שנתיים',
        'default': '{count} שנים'
    },
    'month': {
        1: 'חודש',
        2: 'חודשיים',
        'default': '{count} חודשים'
    },
    'week': {
        1: 'שבוע',
        2: 'שבועיים',
        'default': '{count} שבועות'
    },
    'day': {
        1: 'יום',
        2: 'יומיים',
        'default': '{count} ימים'
    },
    'hour': {
        1: 'שעה',
        2: 'שעתיים',
        'default': '{count} שעות'
    },
    'minute': {
        1: 'דקה',
        2: 'דקותיים',
        'default': '{count} דקות'
    },
    'second': ['שניה', '{count} שניות'],

    # Relative time
    'ago': 'לפני {time}',
    'from_now': 'בעוד {time}',
    'after': 'אחרי {time}',
    'before': 'לפני {time}',

    # Date formats
    'date_formats': {
        'LTS': 'HH:mm:ss',
        'LT': 'HH:mm',
        'LLLL': 'dddd, D [ב]MMMM YYYY HH:mm',
        'LLL': 'D [ב]MMMM YYYY HH:mm',
        'LL': 'D [ב]MMMM YYYY',
        'L': 'DD/MM/YYYY',
    },
}



# -*- coding: utf-8 -*-

translations = {
    # Days
    'days': {
        0: 'domenica',
        1: 'lunedì',
        2: 'martedì',
        3: 'mercoledì',
        4: 'giovedì',
        5: 'venerdì',
        6: 'sabato'
    },
    'days_abbrev': {
        0: 'dom',
        1: 'lun',
        2: 'mar',
        3: 'mer',
        4: 'gio',
        5: 'ven',
        6: 'sab'
    },

    # Months
    'months': {
        1: 'gennaio',
        2: 'febbraio',
        3: 'marzo',
        4: 'aprile',
        5: 'maggio',
        6: 'giugno',
        7: 'luglio',
        8: 'agosto',
        9: 'settembre',
        10: 'ottobre',
        11: 'novembre',
        12: 'dicembre',
    },
    'months_abbrev': {
        1: 'gen',
        2: 'feb',
        3: 'mar',
        4: 'apr',
        5: 'mag',
        6: 'giu',
        7: 'lug',
        8: 'ago',
        9: 'set',
        10: 'ott',
        11: 'nov',
        12: 'dic',
    },

    # Units of time
    'year': ['{count} anno', '{count} anni'],
    'month': ['{count} mese', '{count} mesi'],
    'week': ['{count} settimana', '{count} settimane'],
    'day': ['{count} giorno', '{count} giorni'],
    'hour': ['{count} ora', '{count} ore'],
    'minute': ['{count} minuto', '{count} minuti'],
    'second': ['{count} secondo', '{count} secondi'],

    # Relative time
    'ago': '{time} fa',
    'from_now': '{time} da adesso',
    'after': '{time} dopo',
    'before': '{time} prima',

    # Date formats
    'date_formats': {
        'LTS': 'HH:mm:ss',
        'LT': 'HH:mm',
        'LLLL': 'dddd, D MMMM YYYY HH:mm',
        'LLL': 'D MMMM YYYY HH:mm',
        'LL': 'D MMMM YYYY',
        'L': 'DD/MM/YYYY',
    },
}



# -*- coding: utf-8 -*-

from .af import translations as af_translations
from .ar import translations as ar_translations
from .az import translations as az_translations
from .bg import translations as bg_translations
from .bn import translations as bn_translations
from .ca import translations as ca_translations
from .cs import translations as cs_translations
from .da import translations as da_translations
from .de import translations as de_translations
from .el import translations as el_translations
from .en import translations as en_translations
from .eo import translations as eo_translations
from .es import translations as es_translations
from .et import translations as et_translations
from .eu import translations as eu_translations
from .fa import translations as fa_translations
from .fi import translations as fi_translations
from .fo import translations as fo_translations
from .fr import translations as fr_translations
from .gl import translations as gl_translations
from .he import translations as he_translations
from .hr import translations as hr_translations
from .hu import translations as hu_translations
from .hy import translations as hy_translations
from .id import translations as id_translations
from .it import translations as it_translations
from .ja import translations as ja_translations
from .ka import translations as ka_translations
from .ko import translations as ko_translations
from .lt import translations as lt_translations
from .lv import translations as lv_translations
from .mk import translations as mk_translations
from .ms import translations as ms_translations
from .nl import translations as nl_translations
from .nn import translations as nn_translations
from .pl import translations as pl_translations
from .pt_br import translations as pt_br_translations
from .ro import translations as ro_translations
from .ru import translations as ru_translations
from .sk import translations as sk_translations
from .sl import translations as sl_translations
from .sq import translations as sq_translations
from .sr import translations as sr_translations
from .sv import translations as sv_translations
from .th import translations as th_translations
from .tr import translations as tr_translations
from .uk import translations as uk_translations
from .uz import translations as uz_translations
from .vi import translations as vi_translations
from .zh import translations as zh_translations
from .zh_tw import translations as zh_tw_translations


TRANSLATIONS = {
    "af": af_translations,
    "ar": ar_translations,
    "az": az_translations,
    "bg": bg_translations,
    "bn": bn_translations,
    "ca": ca_translations,
    "cs": cs_translations,
    "da": da_translations,
    "de": de_translations,
    "el": el_translations,
    "en": en_translations,
    "eo": eo_translations,
    "es": es_translations,
    "et": et_translations,
    "eu": eu_translations,
    "fa": fa_translations,
    "fi": fi_translations,
    "fo": fo_translations,
    "fr": fr_translations,
    "gl": gl_translations,
    "he": he_translations,
    "hr": hr_translations,
    "hu": hu_translations,
    "hy": hy_translations,
    "id": id_translations,
    "it": it_translations,
    "ja": ja_translations,
    "ka": ka_translations,
    "ko": ko_translations,
    "lt": lt_translations,
    "lv": lv_translations,
    "mk": mk_translations,
    "ms": ms_translations,
    "nl": nl_translations,
    "nn": nn_translations,
    "pl": pl_translations,
    "pt_br": pt_br_translations,
    "ro": ro_translations,
    "ru": ru_translations,
    "sk": sk_translations,
    "sl": sl_translations,
    "sq": sq_translations,
    "sr": sr_translations,
    "sv": sv_translations,
    "th": th_translations,
    "tr": tr_translations,
    "uk": uk_translations,
    "uz": uz_translations,
    "vi": vi_translations,
    "zh": zh_translations,
    "zh_tw": zh_tw_translations,
}



# -*- coding: utf-8 -*-

translations = {
    # Days
    'days': {
        0: 'неделя',
        1: 'понеделник',
        2: 'вторник',
        3: 'сряда',
        4: 'четвъртък',
        5: 'петък',
        6: 'събота'
    },
    'days_abbrev': {
        0: 'нед',
        1: 'пон',
        2: 'вт',
        3: 'ср',
        4: 'четв',
        5: 'пет',
        6: 'съб'
    },

    # Months
    'months': {
        1: 'януари',
        2: 'февруари',
        3: 'март',
        4: 'април',
        5: 'май',
        6: 'юни',
        7: 'юли',
        8: 'август',
        9: 'септември',
        10: 'октомври',
        11: 'ноември',
        12: 'декември',
    },
    'months_abbrev': {
        1: 'ян',
        2: 'февр',
        3: 'март',
        4: 'апр',
        5: 'май',
        6: 'юни',
        7: 'юли',
        8: 'авг',
        9: 'септ',
        10: 'окт',
        11: 'ноем',
        12: 'дек',
    },

    # Units of time
    'year': ['{count} година', '{count} години'],
    'month': ['{count} месец', '{count} месеца'],
    'week': ['{count} седмица', '{count} седмици'],
    'day': ['{count} ден', '{count} дни'],
    'hour': ['{count} час', '{count} часа'],
    'minute': ['{count} минута', '{count} минути'],
    'second': ['{count} секунда', '{count} секунди'],

    # Relative time
    'ago': 'преди {time}',
    'from_now': '{time} от сега',
    'after': 'след {time}',
    'before': 'преди {time}',

    # Date formats
    'date_formats': {
        'LTS': 'H:mm:ss',
        'LT': 'H:mm',
        'LLLL': 'dddd, D MMMM YYYY H:mm',
        'LLL': 'D MMMM YYYY H:mm',
        'LL': 'D MMMM YYYY',
        'L': 'D.MM.YYYY',
    },
}



# -*- coding: utf-8 -*-

translations = {
    # Days
    'days': {
        0: 'nedelja',
        1: 'ponedeljak',
        2: 'utorak',
        3: 'sreda',
        4: 'četvrtak',
        5: 'petak',
        6: 'subota'
    },
    'days_abbrev': {
        0: 'ned.',
        1: 'pon.',
        2: 'uto.',
        3: 'sre.',
        4: 'čet.',
        5: 'pet.',
        6: 'sub.'
    },

    # Months
    'months': {
        1: 'januar',
        2: 'februar',
        3: 'mart',
        4: 'april',
        5: 'maj',
        6: 'jun',
        7: 'jul',
        8: 'avgust',
        9: 'septembar',
        10: 'oktobar',
        11: 'novembar',
        12: 'decembar'
    },
    'months_abbrev': {
        1: 'jan.',
        2: 'feb.',
        3: 'mar.',
        4: 'apr.',
        5: 'maj',
        6: 'jun',
        7: 'jul',
        8: 'avg.',
        9: 'sep.',
        10: 'okt.',
        11: 'nov.',
        12: 'dec.'
    },

    # Units of time
    'year': ['{count} godina', '{count} godine', '{count} godina'],
    'month': ['{count} mesec', '{count} meseca', '{count} meseci'],
    'week': ['{count} nedelja', '{count} nedelje', '{count} nedelja'],
    'day': ['{count} dan', '{count} dana', '{count} dana'],
    'hour': ['{count} sat', '{count} sata', '{count} sati'],
    'minute': ['{count} minut', '{count} minuta ', '{count} minuta'],
    'second': ['{count} sekund', '{count} sekunde', '{count} sekunde'],

    # Relative time
    'ago': 'pre {time}',
    'from_now': '{time} od sada',
    'after': 'nakon {time}',
    'before': 'pre {time}',

    # Date formats
    'date_formats': {
        'LTS': 'H:mm:ss',
        'LT': 'H:mm',
        'LLLL': 'dddd, D. MMMM YYYY H:mm',
        'LLL': 'D. MMMM YYYY H:mm',
        'LL': 'D. MMMM YYYY',
        'L': 'DD. MM. YYYY',
    },
}



# -*- coding: utf-8 -*-


class TimezoneError(ValueError):

    pass


class NonExistingTime(TimezoneError):

    message = 'The datetime {} does not exist.'

    def __init__(self, dt):
        message = self.message.format(dt)

        super(NonExistingTime, self).__init__(message)


class AmbiguousTime(TimezoneError):

    message = 'The datetime {} is ambiguous.'

    def __init__(self, dt):
        message = self.message.format(dt)

        super(AmbiguousTime, self).__init__(message)
